{
GMClasses class

  ES: clases de apoyo
  EN: support classes

=========================================================================
History:

ver 0.1.8
  ES:
    error: General -> corregidos todos los memory leak encontrados
  EN:
    bug: General -> fixed all memory leaks found

ver 0.1.6:
  ES:
    cambio: GMBase -> se translada a la clase base la funcin GetConvertedString
    nuevo: General -> el proyecto se ha dividido en 2 packages para poder compilar
      en 64 bits en XE2
  EN:
    change: GMBase -> GetConvertedString function is moved to the GMBase base class
    new: General -> the project is divided into 2 packages to compile the project
      into 64 bits under XE2

ver 0.1.3:
  ES:
    cambio: General -> Se ha cambiado la carpeta destino de los DCU. Ahora cada versin tiene la
      suya. Verifica el "library path".
  EN:
    change: General -> Changed destination folder of DCU files. Now, each version have their own
      folder. Check "library path".

ver 0.1:
  ES: primera versin
  EN: first version
=========================================================================
IMPORTANTE PROGRAMADORES: Por favor, si tienes comentarios, mejoras,
  ampliaciones, errores y/o cualquier otro tipo de sugerencia, envame un correo a:
  gmlib@cadetill.com

IMPORTANT PROGRAMMERS: please, if you have comments, improvements, enlargements,
  errors and/or any another type of suggestion, please send me a mail to:
  gmlib@cadetill.com
=========================================================================

Copyright () 2011, by Xavier Martinez (cadetill)

@author Xavier Martinez (cadetill)
@web  http://www.cadetill.com
}
unit GMClasses;

interface

uses
  Classes,
  GMConstants, UAboutFrm, GMLatLng;

type
  { ****************************************************************************
    internal interface
  **************************************************************************** }
  ILinePoint = interface(IInterface)
    ['{35926390-118A-4604-A343-73D200A36006}']
    procedure LinePointChanged;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TLinePoint = class(TCollectionItem)
  private
    FLatLng: TLatLng;
    function GetLat: Real;
    function GetLng: Real;
    procedure SetLat(const Value: Real);
    procedure SetLng(const Value: Real);
  protected
    procedure OnChangeFLatLng(Sender: TObject);
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;

    procedure SetLatLng(LatLng: TLatLng);

    function LatToStr(Precision: Integer = 6): string;
    function LngToStr(Precision: Integer = 6): string;
    function ToStr(Precision: Integer = 6): string;
    function StringToReal(Value: string): Real;
    function GetLatLng: TLatLng;
  published
    property Lat: Real read GetLat write SetLat;
    property Lng: Real read GetLng write SetLng;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TLinePoints = class(TCollection)
  private
    function GetItems(I: Integer): TLinePoint;
    procedure SetItems(I: Integer; const Value: TLinePoint);
  protected
    FOwner: TObject;

    function GetOwner: TPersistent; override;

    property Items[I: Integer]: TLinePoint read GetItems write SetItems; default;
  public
    constructor Create(Owner: TObject; ItemClass: TCollectionItemClass);

    function Add: TLinePoint;
    function Insert(Index: Integer): TLinePoint;
    procedure Delete(Index: Integer);
    procedure Move(CurIndex, NewIndex: Integer);
    procedure Clear;
    function PointsToStr(Precision: Integer = 6): string;

    procedure Assign(Source: TPersistent); override;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TGMBase = class(TComponent)
  private
    FAboutGMLib: TAboutFrm;
    FLanguage: TLang;
  protected
    function GetAPIUrl: string; virtual;
    procedure SetLanguage(const Value: TLang); virtual;
    function ControlPrecision(Value: Real; Prec: Integer{ = 6}): Real; virtual;
    // ES: reemplaza el ' por ~$~&~
    // EN: replace ' by ~$~&~
    function GetConvertedString(Value: string): string;
  public
    constructor Create(AOwner: TComponent); override;

    procedure Assign(Source: TPersistent); override;
  published
    // url a la pgina de la API de Google Maps
    // url to API Google Maps page
    property APIUrl: string read GetAPIUrl;
    // idioma de los mensajes y textos
    // language of messages and texts
    property Language: TLang read FLanguage write SetLanguage;
    // "Realizado por" de los componentes
    // "About box" of the components
    property AboutGMLib: TAboutFrm read FAboutGMLib stored False;
  end;

  { ****************************************************************************
    https://developers.google.com/maps/documentation/javascript/reference?hl=en#Size
  **************************************************************************** }
  TGMSize = class(TPersistent)
  private
    FWidth: Integer;
    FHeight: Integer;
  protected
    procedure SetHeight(const Value: Integer); virtual;
    procedure SetWidth(const Value: Integer); virtual;
  public
    constructor Create; virtual;

    procedure Assign(Source: TPersistent); override;
  published
    property Height: Integer read FHeight write SetHeight;
    property Width: Integer read FWidth write SetWidth;
  end;

implementation

uses
  SysUtils, Math;

{ TLinePoint }

procedure TLinePoint.Assign(Source: TPersistent);
begin
  if Source is TLinePoint then
  begin
    Lat := TLinePoint(Source).Lat;
    Lng := TLinePoint(Source).Lng;
  end
  else
    inherited Assign(Source);
end;

constructor TLinePoint.Create(Collection: TCollection);
begin
  inherited;

  FLatLng := TLatLng.Create;
  FLatLng.OnChange := OnChangeFLatLng;
end;

destructor TLinePoint.Destroy;
begin
  if Assigned(FLatLng) then FreeAndNil(FLatLng);

  inherited;
end;

function TLinePoint.GetLat: Real;
begin
  Result := FLatLng.Lat;
end;

function TLinePoint.GetLatLng: TLatLng;
begin
  Result := FLatLng;
end;

function TLinePoint.GetLng: Real;
begin
  Result := FLatLng.Lng;
end;

function TLinePoint.LatToStr(Precision: Integer): string;
begin
  Result := FLatLng.LatToStr(Precision);
end;

function TLinePoint.LngToStr(Precision: Integer): string;
begin
  Result := FLatLng.LngToStr(Precision);
end;

procedure TLinePoint.OnChangeFLatLng(Sender: TObject);
var
  Intf: ILinePoint;
begin
  if Supports(TLinePoints(Collection).FOwner, ILinePoint, Intf) then
    Intf.LinePointChanged;
end;

procedure TLinePoint.SetLat(const Value: Real);
begin
  if FLatLng.Lat = Value then Exit;

  FLatLng.Lat := Value;
end;

procedure TLinePoint.SetLatLng(LatLng: TLatLng);
begin
  FLatLng.Assign(LatLng);
end;

procedure TLinePoint.SetLng(const Value: Real);
begin
  if FLatLng.Lng = Value then Exit;

  FLatLng.Lng := Value;
end;

function TLinePoint.StringToReal(Value: string): Real;
begin
  Result := FLatLng.StringToReal(Value);
end;

function TLinePoint.ToStr(Precision: Integer): string;
begin
  Result := FLatLng.ToStr(Precision);
end;

{ TLinePoints }

function TLinePoints.Add: TLinePoint;
var
  Intf: ILinePoint;
begin
  Result := TLinePoint(inherited Add);

  if Supports(FOwner, ILinePoint, Intf) then Intf.LinePointChanged;
end;

procedure TLinePoints.Assign(Source: TPersistent);
begin
  inherited;

  if Source is TLinePoints then FOwner := TLinePoints(Source).FOwner;
end;

procedure TLinePoints.Clear;
var
  Intf: ILinePoint;
begin
  inherited Clear;

  if Supports(FOwner, ILinePoint, Intf) then Intf.LinePointChanged;
end;

constructor TLinePoints.Create(Owner: TObject; ItemClass: TCollectionItemClass);
begin
  inherited Create(ItemClass);

  FOwner := Owner;
end;

procedure TLinePoints.Delete(Index: Integer);
var
  Intf: ILinePoint;
begin
  inherited Delete(Index);

  if Supports(FOwner, ILinePoint, Intf) then Intf.LinePointChanged;
end;

function TLinePoints.GetItems(I: Integer): TLinePoint;
begin
  Result := TLinePoint(inherited Items[I]);
end;

function TLinePoints.GetOwner: TPersistent;
begin
  Result := TPersistent(FOwner);
end;

function TLinePoints.Insert(Index: Integer): TLinePoint;
var
  Intf: ILinePoint;
begin
  Result := TLinePoint(inherited Insert(Index));

  if Supports(FOwner, ILinePoint, Intf) then Intf.LinePointChanged;
end;

procedure TLinePoints.Move(CurIndex, NewIndex: Integer);
var
  Intf: ILinePoint;
begin
  Items[CurIndex].Index := NewIndex;

  if Supports(FOwner, ILinePoint, Intf) then Intf.LinePointChanged;
end;

function TLinePoints.PointsToStr(Precision: Integer): string;
var
  i: Integer;
begin
  Result := '';
  for i := 0 to Count - 1 do
    if Result <> '' then
      Result := Result + ';' + Items[i].LatToStr(Precision) + '|' + Items[i].LngToStr(Precision)
    else
      Result := Items[i].LatToStr(Precision) + '|' + Items[i].LngToStr(Precision);
end;

procedure TLinePoints.SetItems(I: Integer; const Value: TLinePoint);
begin
  inherited SetItem(I, Value);
end;

{ TGMBase }

procedure TGMBase.Assign(Source: TPersistent);
begin
  if Source is TGMBase then
  begin
    Language := TGMBase(Source).Language;
    AboutGMLib.Assign(TGMBase(Source).FAboutGMLib);
  end
  else
    inherited Assign(Source);
end;

function TGMBase.ControlPrecision(Value: Real; Prec: Integer): Real;
begin
  Result := Value;

  if Prec > 0 then
    Result := RoundTo(Result, (-1) * Prec);
end;

constructor TGMBase.Create(AOwner: TComponent);
begin
  inherited;

  Language := Espanol;
end;

function TGMBase.GetAPIUrl: string;
begin
  Result := 'https://developers.google.com/maps/documentation/javascript/reference?hl=en';
end;

function TGMBase.GetConvertedString(Value: string): string;
begin
  Result := StringReplace(Value, '''', '~$~&~', [rfReplaceAll]);
end;

procedure TGMBase.SetLanguage(const Value: TLang);
begin
  FLanguage := Value;
end;

{ TGMSize }

procedure TGMSize.Assign(Source: TPersistent);
begin
  if Source is TGMSize then
  begin
    Height := TGMSize(Source).Height;
    Width := TGMSize(Source).Width;
  end
  else inherited;
end;

constructor TGMSize.Create;
begin
  FWidth := 0;
  FHeight := 0;
end;

procedure TGMSize.SetHeight(const Value: Integer);
begin
  if FHeight = Value then Exit;

  FHeight := Value;
end;

procedure TGMSize.SetWidth(const Value: Integer);
begin
  if FWidth = Value then Exit;

  FWidth := Value;
end;

end.
