{
TGMMarker component

  ES: componente para mostrar ventanas de informacin en un mapa de Google Maps
      mediante el componente TGMMap
  EN: component to show balloons with information on Google Map map using the
      component TGMMap

=========================================================================
MODO DE USO/HOW TO USE

  ES: poner el componente en el formulario, linkarlo a un TGMMap y poner el
      texto a mostrar en la propiedad "HTMLContent"
  EN: put the component into a form, link to a TGMMap and put the text to show
      into "HTMLContent" property
=========================================================================
History:

ver 0.1.1:
  ES:
    nuevo: aadida propiedad AutoOpen en la clase TInfoWindow
    nuevo: aadido evento OnPositionChange
  EN:
    new: added property AutoOpen in TInfoWindow class
    new: added event OnPositionChange

ver 0.1:
  ES- primera versin
  EN- first version
=========================================================================
IMPORTANTE PROGRAMADORES: Por favor, si tienes comentarios, mejoras,
  ampliaciones, errores y/o cualquier otro tipo de sugerencia, envame un correo a:
  gmlib@cadetill.com

IMPORTANT PROGRAMMERS: please, if you have comments, improvements, enlargements,
  errors and/or any another type of suggestion, please send me a mail to:
  gmlib@cadetill.com
=========================================================================

Copyright () 2012, by Xavier Martinez (cadetill)

@author Xavier Martinez (cadetill)
@web  http://www.cadetill.com
}
unit GMInfoWindow;

interface

uses
  Classes,
  GMLatLng, GMMap, GMBaseInfoWindow;

type
  TGMInfoWindow = class;
  TInfoWindow = class;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TIWLatLng = class(TLatLng)
  private
    FInfoWindow: TInfoWindow;
    FIdxList: Cardinal;
    procedure PositionChange;
  protected
    procedure SetLat(const Value: Real); override;
    procedure SetLng(const Value: Real); override;

    property IdxList: Cardinal read FIdxList write FIdxList;
  public
    constructor Create(InfoWindow: TInfoWindow; Lat: Real = 0; Lng: Real = 0); reintroduce; virtual;

    procedure Assign(Source: TPersistent); override;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TInfoWindow = class(TLinkedComponent)
  private
    FPosition: TIWLatLng;
    FAutoOpen: Boolean;
    function GetContent: string;
    function GetDisableAutoPan: Boolean;
    function GetMaxWidth: Integer;
    function GetPixelOffset: TIWSize;
    function GetCloseOtherBeforeOpen: Boolean;
    procedure SetContent(const Value: string);
    procedure SetDisableAutoPan(const Value: Boolean);
    procedure SetMaxWidth(const Value: Integer);
    procedure SetPixelOffset(const Value: TIWSize);
    procedure SetCloseOtherBeforeOpen(const Value: Boolean);
  protected
    procedure SetIdxList(const Value: Cardinal); override;

    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;

    procedure CenterMapTo;
    procedure OpenClose;
  published
    property HTMLContent: string read GetContent write SetContent;
    property DisableAutoPan: Boolean read GetDisableAutoPan write SetDisableAutoPan;
    property MaxWidth: Integer read GetMaxWidth write SetMaxWidth;  // 0 = no MaxWidth
    property PixelOffset: TIWSize read GetPixelOffset write SetPixelOffset; // (0,0) = no pixelOffset
    property CloseOtherBeforeOpen: Boolean read GetCloseOtherBeforeOpen write SetCloseOtherBeforeOpen;
    property Position: TIWLatLng read FPosition write FPosition;
    property AutoOpen: Boolean read FAutoOpen write FAutoOpen;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TInfoWindows = class(TLinkedComponents)
  private
    function GetItems(I: Integer): TInfoWindow;
    procedure SetItems(I: Integer; const Value: TInfoWindow);
  protected
    function GetOwner: TPersistent; override;
  public
    function Add: TInfoWindow;
    function Insert(Index: Integer): TInfoWindow;

    property Items[I: Integer]: TInfoWindow read GetItems write SetItems; default;
  end;

  { ****************************************************************************
    https://developers.google.com/maps/documentation/javascript/reference?hl=en#InfoWindow
  **************************************************************************** }
  TGMInfoWindow = class(TGMLinkedComponent)
  private
    FOnPositionChange: TLatLngIdxEvent;
  protected
    function GetAPIUrl: string; override;

    function GetItems(I: Integer): TInfoWindow;

    procedure ShowElements; override;
    function GetCollectionItemClass: TLinkedComponentClass; override;
    function GetCollectionClass: TLinkedComponentsClass; override;
  public
    function Add(Lat: Real = 0; Lng: Real = 0; HTMLContent: string = ''): TInfoWindow;

    property Items[I: Integer]: TInfoWindow read GetItems; default;
  published
    property VisualObjects;
    property OnPositionChange: TLatLngIdxEvent read FOnPositionChange write FOnPositionChange;
  end;

implementation

uses
  SysUtils;

{ TGMInfoWindow }

function TGMInfoWindow.Add(Lat, Lng: Real; HTMLContent: string): TInfoWindow;
begin
  Result := TInfoWindow(inherited Add);
  Result.Position.Lat := Lat;
  Result.Position.Lng := Lng;
  Result.HTMLContent := HTMLContent;
end;

function TGMInfoWindow.GetAPIUrl: string;
begin
  Result := 'https://developers.google.com/maps/documentation/javascript/reference?hl=en#InfoWindow';
end;

function TGMInfoWindow.GetCollectionClass: TLinkedComponentsClass;
begin
  Result := TInfoWindows;
end;

function TGMInfoWindow.GetCollectionItemClass: TLinkedComponentClass;
begin
  Result := TInfoWindow;
end;

function TGMInfoWindow.GetItems(I: Integer): TInfoWindow;
begin
  Result := TInfoWindow(inherited Items[i]);
end;

procedure TGMInfoWindow.ShowElements;
const
  StrParams = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s';
var
  Params: string;
  i: Integer;
begin
  if not ExecuteScript('DeleteObjects', IntToStr(IdxList)) then Exit;

  for i:= 0 to VisualObjects.Count - 1 do
  begin
    Params := Format(StrParams, [
                    IntToStr(IdxList),
                    IntToStr(Items[i].Index),
                    QuotedStr(Items[i].InfoWindow.GetConvertedString),
                    LowerCase(BoolToStr(Items[i].DisableAutoPan, True)),
                    IntToStr(Items[i].MaxWidth),
                    IntToStr(Items[i].PixelOffset.Height),
                    IntToStr(Items[i].PixelOffset.Width),
                    Items[i].Position.LatToStr(Map.Precision),
                    Items[i].Position.LngToStr(Map.Precision),
                    LowerCase(BoolToStr(Items[i].CloseOtherBeforeOpen, True))
                    ]);

    ExecuteScript('MakeInfoWindow', Params);

    if Items[i].AutoOpen then Items[i].OpenClose;
  end;
end;

{ TInfoWindows }

function TInfoWindows.Add: TInfoWindow;
begin
  Result := TInfoWindow(inherited Add);
end;

function TInfoWindows.GetItems(I: Integer): TInfoWindow;
begin
  Result := TInfoWindow(inherited Items[I]);
end;

function TInfoWindows.GetOwner: TPersistent;
begin
  Result := TGMInfoWindow(inherited GetOwner);
end;

function TInfoWindows.Insert(Index: Integer): TInfoWindow;
begin
  Result := TInfoWindow(inherited Insert(Index));
end;

procedure TInfoWindows.SetItems(I: Integer; const Value: TInfoWindow);
begin
  inherited SetItem(I, Value);
end;

{ TInfoWindow }

procedure TInfoWindow.Assign(Source: TPersistent);
begin
  if Source is TInfoWindow then
  begin
    Position.Assign(TInfoWindow(Source).Position);
    AutoOpen := TInfoWindow(Source).AutoOpen;
  end
  else
    inherited Assign(Source);
end;

procedure TInfoWindow.CenterMapTo;
begin
  if Assigned(Collection) and (Collection is TInfoWindows) and
     Assigned(TInfoWindows(Collection).FGMLinkedComponent) and
     Assigned(TInfoWindows(Collection).FGMLinkedComponent.Map) then
    TInfoWindows(Collection).FGMLinkedComponent.Map.SetCenter(Position.Lat, Position.Lng);
end;

constructor TInfoWindow.Create(Collection: TCollection);
begin
  inherited;

  FPosition := TIWLatLng.Create(Self);
  FAutoOpen := True;
end;

destructor TInfoWindow.Destroy;
begin
  if Assigned(FPosition) then FreeAndNil(FPosition);

  inherited;
end;

function TInfoWindow.GetCloseOtherBeforeOpen: Boolean;
begin
  Result := InfoWindow.CloseOtherBeforeOpen;
end;

function TInfoWindow.GetContent: string;
begin
  Result := InfoWindow.HTMLContent;
end;

function TInfoWindow.GetDisableAutoPan: Boolean;
begin
  Result := InfoWindow.DisableAutoPan;
end;

function TInfoWindow.GetDisplayName: string;
begin
  if Length(InfoWindow.HTMLContent) > 0 then
  begin
    if Length(InfoWindow.HTMLContent) > 15 then
      Result := Copy(InfoWindow.HTMLContent, 0, 12) + '...'
    else
      Result := InfoWindow.HTMLContent;
  end
  else
    Result := inherited GetDisplayName;
end;

function TInfoWindow.GetMaxWidth: Integer;
begin
  Result := InfoWindow.MaxWidth;
end;

function TInfoWindow.GetPixelOffset: TIWSize;
begin
  Result := InfoWindow.PixelOffset;
end;

procedure TInfoWindow.OpenClose;
const
  StrParams = 'null,%s,%s';
var
  Params: string;
begin
  if Assigned(Collection) and (Collection is TInfoWindows) and
     Assigned(TInfoWindows(Collection).FGMLinkedComponent) and
     TGMInfoWindow(TInfoWindows(Collection).FGMLinkedComponent).AutoUpdate then
  begin
    Params := Format(StrParams, [
                      IntToStr(IdxList),
                      IntToStr(Index)]);
    TGMInfoWindow(TInfoWindows(Collection).FGMLinkedComponent).ExecuteScript('InfoWindowOpenClose', Params);
  end;
end;

procedure TInfoWindow.SetCloseOtherBeforeOpen(const Value: Boolean);
begin
  InfoWindow.CloseOtherBeforeOpen := Value;
end;

procedure TInfoWindow.SetContent(const Value: string);
begin
  InfoWindow.HTMLContent := Value;
end;

procedure TInfoWindow.SetDisableAutoPan(const Value: Boolean);
begin
  InfoWindow.DisableAutoPan := Value;
end;

procedure TInfoWindow.SetIdxList(const Value: Cardinal);
begin
  inherited;

  FPosition.IdxList := Value;
end;

procedure TInfoWindow.SetMaxWidth(const Value: Integer);
begin
  InfoWindow.MaxWidth := Value;
end;

procedure TInfoWindow.SetPixelOffset(const Value: TIWSize);
begin
  InfoWindow.PixelOffset := Value;
end;

{ TIWLatLng }

procedure TIWLatLng.Assign(Source: TPersistent);
begin
  inherited;

  if Source is TIWLatLng then
    IdxList := TIWLatLng(Source).IdxList;
end;

constructor TIWLatLng.Create(InfoWindow: TInfoWindow; Lat, Lng: Real);
begin
  inherited Create(Lat, Lng);

  FInfoWindow := InfoWindow;
  if Assigned(FInfoWindow) then FIdxList := FInfoWindow.IdxList;
end;

procedure TIWLatLng.PositionChange;
const
  StrParams = '%s,%s,%s,%s';
var
  Params: String;
begin
  if Assigned(FInfoWindow) and (FInfoWindow is TInfoWindow) and
     Assigned(FInfoWindow.Collection) and (FInfoWindow.Collection is TInfoWindows) and
     Assigned(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent) and
     TGMInfoWindow(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent).AutoUpdate and
     Assigned(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent.Map) then
  begin
    Params := Format(StrParams, [
                   IntToStr(FIdxList),
                   FInfoWindow.Position.LatToStr(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent.Map.Precision),
                   FInfoWindow.Position.LngToStr(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent.Map.Precision),
                   IntToStr(FInfoWindow.Index)
                  ]);
    TGMInfoWindow(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent).ExecuteScript('InfoWindowSetPosition', Params);

    // ES/EN: evento/event
    if Assigned(TGMInfoWindow(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent).OnPositionChange) then
      TGMInfoWindow(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent).OnPositionChange(TGMInfoWindow(TInfoWindows(FInfoWindow.Collection).FGMLinkedComponent), Self, FInfoWindow.Index, FInfoWindow);
  end;
end;

procedure TIWLatLng.SetLat(const Value: Real);
begin
  if Lat = Value then Exit;

  inherited;

  PositionChange;
end;

procedure TIWLatLng.SetLng(const Value: Real);
begin
  if Lng = Value then Exit;

  inherited;

  PositionChange;
end;

end.
