{
TLatLng class

  ES: LatLng es un punto en las coordenadas geogrficas, latitud y longitud
  EN: LatLng is a point in geographical coordinates, latitude and longitude

=========================================================================
History:

ver 0.1.5
  ES:
    - error: corregido error en la funcin LatLngToStr (gracias Erasmo)
    - cambio: se controla la posible excepcin en la funcin StringToReal
  EN:
    - bug: fix error in LatLngToStr function (thanks Erasmo)
    - change: it is controlled a possible exception in function StringToReal

ver 0.1.4:
  ES:
    - error: corregido error en la funcin LatLngToStr (gracias Shaun)
  EN:
    - bug: fix error in LatLngToStr function (thanks Shaun)

ver 0.1.3:
  ES:
    - nuevo: aadidos evento OnChange
    - nuevo: aadido mtodo StringToReal
  EN:
    - new: added OnChange event
    - new: added StringToReal method

ver 0.1:
  ES: primera versin
  EN: first version
=========================================================================
IMPORTANTE PROGRAMADORES: Por favor, si tienes comentarios, mejoras,
  ampliaciones, errores y/o cualquier otro tipo de sugerencia, envame un correo a:
  gmlib@cadetill.com

IMPORTANT PROGRAMMERS: please, if you have comments, improvements, enlargements,
  errors and/or any another type of suggestion, please send me a mail to:
  gmlib@cadetill.com
=========================================================================

Copyright () 2011, by Xavier Martinez (cadetill)

@author Xavier Martinez (cadetill)
@web  http://www.cadetill.com
}
unit GMLatLng;

interface

uses
  Classes;

type
  { ****************************************************************************
    https://developers.google.com/maps/documentation/javascript/reference?hl=en#LatLng
  **************************************************************************** }
  TLatLng = class(TPersistent)
  private
    FLat: Real;
    FLng: Real;
    FOnChange: TNotifyEvent;
  protected
    procedure SetLat(const Value: Real); virtual;
    procedure SetLng(const Value: Real); virtual;

    procedure LatLngToStr(var aLat, aLng: string; Precision: Integer = 6);
  public
    constructor Create(Lat: Real = 0; Lng: Real = 0); virtual;

    procedure Assign(Source: TPersistent); override;

    function IsEqual(Other: TLatLng): Boolean; virtual;
    function ToStr(Precision: Integer = 6): string;
    function ToUrlValue(Precision: Integer = 6): string;
    function LatToStr(Precision: Integer = 6): string;
    function LngToStr(Precision: Integer = 6): string;
    function StringToReal(Value: string): Real;

    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property Lat: Real read FLat write SetLat;
    property Lng: Real read FLng write SetLng;
  end;

implementation

uses
  SysUtils;

{ TLatLng }

procedure TLatLng.Assign(Source: TPersistent);
begin
  if Source is TLatLng then
  begin
    Lat := TLatLng(Source).Lat;
    Lng := TLatLng(Source).Lng;
  end
  else
    inherited Assign(Source);
end;

constructor TLatLng.Create(Lat, Lng: Real);
begin
  FLat := Lat;
  FLng := Lng;
end;

function TLatLng.IsEqual(Other: TLatLng): Boolean;
begin
  Result := (FLat = Other.Lat) and (FLng = Other.Lng);
end;

procedure TLatLng.LatLngToStr(var aLat, aLng: string; Precision: Integer);
var
  i: Integer;
  Prec: Integer;
begin
  if Precision > 0 then
  begin
    Prec := 1;
    for i := 1 to Precision do Prec := Prec * 10;
    aLat := FloatToStr(Trunc(FLat * Prec) / Prec);
    aLng := FloatToStr(Trunc(FLng * Prec) / Prec);
  end
  else
  begin
    aLat := FloatToStr(FLat);
    aLng := FloatToStr(FLng);
  end;

  if {$IF CompilerVersion < 23}DecimalSeparator{$ELSE}FormatSettings.DecimalSeparator{$IFEND} = ',' then
  begin
    aLat := StringReplace(aLat, ',', '.', [rfReplaceAll]);
    aLng := StringReplace(aLng, ',', '.', [rfReplaceAll]);
  end;
end;

function TLatLng.LatToStr(Precision: Integer): string;
var
  La, Ln: string;
begin
  LatLngToStr(La, Ln, Precision);
  Result := La;
end;

function TLatLng.LngToStr(Precision: Integer): string;
var
  La, Ln: string;
begin
  LatLngToStr(La, Ln, Precision);
  Result := Ln;
end;

procedure TLatLng.SetLat(const Value: Real);
begin
  if FLat = Value then Exit;

  FLat := Value;
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TLatLng.SetLng(const Value: Real);
begin
  if FLng = Value then Exit;

  FLng := Value;
  if Assigned(FOnChange) then FOnChange(Self);
end;

function TLatLng.StringToReal(Value: string): Real;
begin
  if {$IF CompilerVersion < 23}DecimalSeparator{$ELSE}FormatSettings.DecimalSeparator{$IFEND} = ',' then
    Value := StringReplace(Value, '.', ',', [rfReplaceAll]);
  try
    Result := StrToFloat(Value);
  except
    on E: Exception do
      raise Exception.Create(Value + ' is not a valid real value');
  end;
end;

function TLatLng.ToStr(Precision: Integer): string;
const
  Str = '(%s, %s)';
var
  La, Ln: string;
begin
  LatLngToStr(La, Ln, Precision);
  Result := Format(Str, [La, Ln]);
end;

function TLatLng.ToUrlValue(Precision: Integer): string;
const
  Str = '%s,%s';
var
  La, Ln: string;
begin
  LatLngToStr(La, Ln, Precision);
  Result := Format(Str, [La, Ln]);
end;

end.
