{
TGMMarker component

  ES: componente para mostrar marcadores en un mapa de Google Maps mediante el
      componente TGMMap
  EN: component to show markers on Google Map map using the component TGMMap

=========================================================================
MODO DE USO/HOW TO USE

  ES: poner el componente en el formulario, linkarlo a un TGMMap y poner los
      marcadores a mostrar
  EN: put the component into a form, link to a TGMMap and put the markers to show
=========================================================================
History:

ver 0.1.7
  ES:
    cambio: modificados mtodos Set y ShowElement para que usen el nuevo mtodo
      ChangeProperties heredado de TLinkedComponent
    cambio: suprimida clase TMarkerLatLng. Ahora se usa directamente la clase TLatLng
    cambio: TMarker-> la pripiedad Position ahora es de tipo TLatLng.
    cambio: TMarker-> aadida propiedad MarkerType para especificar el tipo de
      marcador a mostrar (mtStandard, mtColored, mtStyledMarker).
    cambio: TMarker-> aadida propiedad ColoredMarker para especificar la forma
      del marcador si MarkerType = mtColored.
    cambio: TMarker-> aadida propiedad StyledMarker para especificar la forma
      del marcador si MarkerType = mtStyledMarker.
    cambio: TGMMarker-> aadido mtodo pblico LoadFromCSV para una carga masiva
      de marcadores que estn en un archivo CSV (unos 500 cada 10 seg.).
    cambio: TGMMarker-> 4 nuevos eventos: OnLoadFile y AfterLoadFile para el procesado
      del fichero CSV, y OnColoredMarkerChange y OnStyledMarkerChange para el cambio
      de las propiedades
  EN:
    change: modified all Set and ShowElements methods to use the new method
      ChangeProperties inherited from TLinkedComponent
    change: TMarkerLatLng class are deleted. Now it use the TLatLng class directly
    change: TMarker-> the Position property now is a TLatLng.
    change: TMarker-> added MarkerType property to specify the marker type to
      show (mtStandard, mtColored, mtStyledMarker).
    change: TMarker-> added ColoredMarker property to specify properties of marker
      when MarkerType = mtColored.
    change: TMarker-> added StyledMarker property to specify properties of marker
      when MarkerType = mtStyledMarker.
    change: TGMMarker-> added LoadFromCSV public method for a massive load of
      markers that are in a CSV file (about 500 each 10 sec).
    change: TGMMarker-> 4 news events: OnLoadFile and AfterLoadFile for processing
      the CSV file, and OnColoredMarkerChange and OnStyledMarkerChange for changing
      the properties

ver 0.1.5
  ES:
    error: se controlan la comilla simple en la propiedad Title
  EN:
    bug: the single quote is controled into Title property

ver 0.1.4
  ES:
    cambio: cambio en el JavaScript de algunos mtodos Set
  EN:
    change: JavaScript changed from some Set methods

ver 0.1.3
  ES:
    nuevo: aadido propiedad Icon para especificar la imagen del marcador. Puede
      ser una imagen en la web (url) o una imagen en el ordenador. En blanco ser
      la imagen por defecto.
    nuevo: aadido evento OnIconChange
    cambio: mtodos Set cambiados para evitar duplicidad de cdigo
  EN:
    new: added Icon property to specify a marker image. You can specify a web
      image (url) or a file in the PC. Nothing for default image
    new: added OnIconChange event
    change: changed Set methods to avoid duplicate code

ver 0.1.2
  ES:
    error: algunas propiedades daban un error de JavaScript al modificarse
  EN:
    bug: some properties giving a JavaScript error when modifying

ver 0.1.1
  ES:
    nuevo: Aadido evento Assign a la clase TMarkerLatLng
    cambio: Cuando se cambia la lat/lng de un marcador, ste cambia automticamente en el mapa
  EN:
    new: Added Assign Assign to the TMarkerLatLng class
    change: When a lat/lng is changed into a marker, this is automatically changed into the map

ver 0.1:
  ES- primera versin
  EN- first version
=========================================================================
IMPORTANTE PROGRAMADORES: Por favor, si tienes comentarios, mejoras,
  ampliaciones, errores y/o cualquier otro tipo de sugerencia, envame un correo a:
  gmlib@cadetill.com

IMPORTANT PROGRAMMERS: please, if you have comments, improvements, enlargements,
  errors and/or any another type of suggestion, please send me a mail to:
  gmlib@cadetill.com
=========================================================================

Copyright () 2012, by Xavier Martinez (cadetill)

@author Xavier Martinez (cadetill)
@web  http://www.cadetill.com
}
unit GMMarker;

interface

uses
  Classes, Graphics, Types,
  GMMap, GMLatLng, GMConstants;

const
  StrColored = 'http://chart.apis.google.com/chart?cht=mm&chs=%dx%d&chco=%s,%s,%s&ext=.png';

type
  TGMMarker = class;
  TMarker = class;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TAnimation = class(TPersistent)
  private
    FMarker: TMarker;
    FOnDrop: Boolean;
    FBounce: Boolean;
    FIdxList: Cardinal;
    procedure SetOnDrop(const Value: Boolean);
  protected
    procedure SetBounce(const Value: Boolean); virtual;

    property IdxList: Cardinal read FIdxList write FIdxList;
  public
    constructor Create(aOwner: TMarker); virtual;

    procedure Assign(Source: TPersistent); override;
  published
    property OnDrop: Boolean read FOnDrop write SetOnDrop;
    property Bounce: Boolean read FBounce write SetBounce;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TColoredMarker = class(TPersistent)
  private
    FMarker: TMarker;
    FWidth: Integer;
    FStrokeColor: TColor;
    FCornerColor: TColor;
    FPrimaryColor: TColor;
    FHeight: Integer;
    procedure SetCornerColor(const Value: TColor);
    procedure SetHeight(const Value: Integer);
    procedure SetPrimaryColor(const Value: TColor);
    procedure SetStrokeColor(const Value: TColor);
    procedure SetWidth(const Value: Integer);
  public
    constructor Create(aOwner: TMarker); virtual;

    procedure Assign(Source: TPersistent); override;
  published
    property Width: Integer read FWidth write SetWidth;
    property Height: Integer read FHeight write SetHeight;
    property PrimaryColor: TColor read FPrimaryColor write SetPrimaryColor;
    property StrokeColor: TColor read FStrokeColor write SetStrokeColor;
    property CornerColor: TColor read FCornerColor write SetCornerColor;
  end;

  TStyledMarker = class(TPersistent)
  private
    FMarker: TMarker;
    FStarColor: TColor;
    FTextColor: TColor;
    FShowStar: Boolean;
    FStyledIcon: TStyledIcon;
    FBackgroundColor: TColor;
    procedure SetBackgroundColor(const Value: TColor);
    procedure SetShowStar(const Value: Boolean);
    procedure SetStarColor(const Value: TColor);
    procedure SetStyledIcon(const Value: TStyledIcon);
    procedure SetTextColor(const Value: TColor);
  public
    constructor Create(aOwner: TMarker); virtual;

    procedure Assign(Source: TPersistent); override;
  published
    property StyledIcon: TStyledIcon read FStyledIcon write SetStyledIcon;
    property BackgroundColor: TColor read FBackgroundColor write SetBackgroundColor;
    property TextColor: TColor read FTextColor write SetTextColor;
    property StarColor: TColor read FStarColor write SetStarColor;
    property ShowStar: Boolean read FShowStar write SetShowStar;
  end;

  TGMFontStyle = (fsBold, fsItalic, fsOverline, fsUnderline, fsStrikeOut);
  TGMFontStyles = set of TGMFontStyle;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TGMBorder = class(TPersistent)
  private
    FMarker: TMarker;
    FColor: TColor;
    FSize: Integer;
    FStyle: TBorderStyle;
    procedure SetColor(const Value: TColor);
    procedure SetSize(const Value: Integer);
    procedure SetStyle(const Value: TBorderStyle);
  public
    constructor Create(aOwner: TMarker); virtual;

    procedure Assign(Source: TPersistent); override;
  published
    property Size: Integer read FSize write SetSize;
    property Color: TColor read FColor write SetColor;
    property Style: TBorderStyle read FStyle write SetStyle;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TGMFont = class(TPersistent)
  private
    FMarker: TMarker;
    FColor: TColor;
    FSize: Integer;
    FStyle: TGMFontStyles;
    procedure SetColor(const Value: TColor);
    procedure SetSize(const Value: Integer);
    procedure SetStyle(const Value: TGMFontStyles);
  public
    constructor Create(aOwner: TMarker); virtual;

    procedure Assign(Source: TPersistent); override;
  published
    property Style: TGMFontStyles read FStyle write SetStyle;
    property Size: Integer read FSize write SetSize;
    property Color: TColor read FColor write SetColor;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TStyleLabel = class(TPersistent)
  private
    FMarker: TMarker;
    FFont: TGMFont;
    FBorder: TGMBorder;
    FOpacity: Real;
    FBackground: TColor;
    FNoBackground: Boolean;
    procedure SetOpacity(const Value: Real);
    procedure SetBackground(const Value: TColor);
    procedure SetNoBackground(const Value: Boolean);
  public
    constructor Create(aOwner: TMarker); virtual;
    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;
  published
    property Font: TGMFont read FFont write FFont;
    property Border: TGMBorder read FBorder write FBorder;
    property Opacity: Real read FOpacity write SetOpacity;
    property Background: TColor read FBackground write SetBackground;
    property NoBackground: Boolean read FNoBackground write SetNoBackground;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TMarker = class(TLinkedComponent)
  private
    FAnimation: TAnimation;
    FClickable: Boolean;
    FDraggable: Boolean;
    FFlat: Boolean;
    FOptimized: Boolean;
    FPosition: TLatLng;
    FRaiseOnDrag: Boolean;
    FTitle: string;
    FVisible: Boolean;
    FIcon: string;
    FIsUpdating: Boolean;
    FMarkerType: TMarkerType;
    FColoredMarker: TColoredMarker;
    FStyledMarker: TStyledMarker;

    procedure SetClickable(const Value: Boolean);
    procedure SetDraggable(const Value: Boolean);
    procedure SetFlat(const Value: Boolean);
    procedure SetRaiseOnDrag(const Value: Boolean);
    procedure SetVisible(const Value: Boolean);
    function GetZIndex: Integer;
    procedure SetOptimized(const Value: Boolean);
    procedure SetTitle(const Value: string);
    procedure SetIcon(const Value: string);
    procedure OnLatLngChange(Sender: TObject);
    procedure SetMarkerType(const Value: TMarkerType);
  protected
    procedure SetIdxList(const Value: Cardinal); override;

    function GetDisplayName: string; override;
    function ChangeProperties: Boolean; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;

    procedure Assign(Source: TPersistent); override;

    procedure CenterMapToMarker;
  published
    property MarkerType: TMarkerType read FMarkerType write SetMarkerType;
    property Animation: TAnimation read FAnimation write FAnimation;
    property Clickable: Boolean read FClickable write SetClickable;  // slo regenera el propio Marker
    property Draggable: Boolean read FDraggable write SetDraggable;  // slo regenera el propio Marker
    property Flat: Boolean read FFlat write SetFlat;                 // slo regenera el propio Marker
    property Position: TLatLng read FPosition write FPosition;       // slo regenera el propio Marker
    property Title: string read FTitle write SetTitle;               // slo regenera el propio Marker
    property Visible: Boolean read FVisible write SetVisible;        // slo regenera el propio Marker
    property Optimized: Boolean read FOptimized write SetOptimized;       // regenera todos los Markers
    property RaiseOnDrag: Boolean read FRaiseOnDrag write SetRaiseOnDrag; // regenera todos los Markers
    property Icon: string read FIcon write SetIcon;
    property ZIndex: Integer read GetZIndex;
    property InfoWindow;
    property ShowInfoWinMouseOver;
    property ColoredMarker: TColoredMarker read FColoredMarker write FColoredMarker;
    property StyledMarker: TStyledMarker read FStyledMarker write FStyledMarker;
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TMarkers = class(TLinkedComponents)
  private
    procedure SetItems(I: Integer; const Value: TMarker);
    function GetItems(I: Integer): TMarker;
  protected
    function GetOwner: TPersistent; override;
  public
    function Add: TMarker;
    function Insert(Index: Integer): TMarker;

    property Items[I: Integer]: TMarker read GetItems write SetItems; default;
  end;

  TOnLoadFile = procedure (Sender: TGMMarker; Marker: TMarker; Current, Count: Integer; var Stop: Boolean) of object;
  TAfterLoadFile = procedure (Sender: TGMMarker; Loaded, Count: Integer) of object;
  { ****************************************************************************
    https://developers.google.com/maps/documentation/javascript/reference?hl=en#Marker
  **************************************************************************** }
  TGMMarker  = class(TGMLinkedComponent)
  private
    FOnClick: TLatLngIdxEvent;
    FOnDblClick: TLatLngIdxEvent;
    FOnDrag: TLatLngIdxEvent;
    FOnDragEnd: TLatLngIdxEvent;
    FOnRightClick: TLatLngIdxEvent;
    FOnMouseDown: TLatLngIdxEvent;
    FOnMouseUp: TLatLngIdxEvent;
    FOnMouseOut: TLatLngIdxEvent;
    FOnDragStart: TLatLngIdxEvent;
    FOnMouseOver: TLatLngIdxEvent;
    FOnClickableChange: TLinkedComponentChange;
    FOnDragableChange: TLinkedComponentChange;
    FOnFlatChange: TLinkedComponentChange;
    FOnPositionChange: TLatLngIdxEvent;
    FOnTitleChange: TLinkedComponentChange;
    FOnVisibleChange: TLinkedComponentChange;
    FOnIconChange: TLinkedComponentChange;
    FOnLoadFile: TOnLoadFile;
    FAfterLoadFile: TAfterLoadFile;
    FOnColoredMarkerChange: TLinkedComponentChange;
    FOnStyledMarkerChange: TLinkedComponentChange;
  protected
    function GetAPIUrl: string; override;

    function GetItems(I: Integer): TMarker;

    procedure EventFired(EventType: TEventType; Params: array of const); override;
    function GetCollectionItemClass: TLinkedComponentClass; override;
    function GetCollectionClass: TLinkedComponentsClass; override;
  public
    procedure ShowElements; override;
    procedure LoadFromCSV(LatColumn, LngColumn: Integer; FileName: string;
      TitleColumn: Integer = -1; Delimiter: Char = ','; DeleteBeforeLoad: Boolean = True;
      WithRownTitle: Boolean = True);

    function Add(Lat: Real = 0; Lng: Real = 0; Title: string = ''): TMarker;

    property Items[I: Integer]: TMarker read GetItems; default;
  published
    property VisualObjects;
    // eventos
    // events
    property OnLoadFile: TOnLoadFile read FOnLoadFile write FOnLoadFile;
    property AfterLoadFile: TAfterLoadFile read FAfterLoadFile write FAfterLoadFile;
        // from map
    property OnClick: TLatLngIdxEvent read FOnClick write FOnClick;
    property OnDblClick: TLatLngIdxEvent read FOnDblClick write FOnDblClick;
    property OnDrag: TLatLngIdxEvent read FOnDrag write FOnDrag;
    property OnDragEnd: TLatLngIdxEvent read FOnDragEnd write FOnDragEnd;
    property OnDragStart: TLatLngIdxEvent read FOnDragStart write FOnDragStart;
    property OnMouseDown: TLatLngIdxEvent read FOnMouseDown write FOnMouseDown;
    property OnMouseOut: TLatLngIdxEvent read FOnMouseOut write FOnMouseOut;
    property OnMouseOver: TLatLngIdxEvent read FOnMouseOver write FOnMouseOver;
    property OnMouseUp: TLatLngIdxEvent read FOnMouseUp write FOnMouseUp;
    property OnRightClick: TLatLngIdxEvent read FOnRightClick write FOnRightClick;
        // from properties
    property OnClickableChange: TLinkedComponentChange read FOnClickableChange write FOnClickableChange;
    property OnDragableChange: TLinkedComponentChange read FOnDragableChange write FOnDragableChange;
    property OnFlatChange: TLinkedComponentChange read FOnFlatChange write FOnFlatChange;
    property OnPositionChange: TLatLngIdxEvent read FOnPositionChange write FOnPositionChange;
    property OnTitleChange: TLinkedComponentChange read FOnTitleChange write FOnTitleChange;
    property OnVisibleChange: TLinkedComponentChange read FOnVisibleChange write FOnVisibleChange;
    property OnIconChange: TLinkedComponentChange read FOnIconChange write FOnIconChange;
    property OnColoredMarkerChange: TLinkedComponentChange read FOnColoredMarkerChange write FOnColoredMarkerChange;
    property OnStyledMarkerChange: TLinkedComponentChange read FOnStyledMarkerChange write FOnStyledMarkerChange;
  end;

implementation

uses
  SysUtils,
  WebControl, Lang, GMFunctions;

{ TGMMarker }

function TGMMarker.Add(Lat, Lng: Real; Title: string): TMarker;
begin
  Result := TMarker(inherited Add);
  Result.Position.Lat := Lat;
  Result.Position.Lng := Lng;
  if Title <> '' then Result.Title := Title
  else Result.Title := Result.GetDisplayName;
end;

function TGMMarker.GetAPIUrl: string;
begin
  Result := 'https://developers.google.com/maps/documentation/javascript/reference?hl=en#Marker';
end;

function TGMMarker.GetCollectionClass: TLinkedComponentsClass;
begin
  Result := TMarkers;
end;

function TGMMarker.GetCollectionItemClass: TLinkedComponentClass;
begin
  Result := TMarker;
end;

function TGMMarker.GetItems(I: Integer): TMarker;
begin
  Result := TMarker(inherited Items[i]);
end;

procedure TGMMarker.LoadFromCSV(LatColumn, LngColumn: Integer; FileName: string;
  TitleColumn: Integer; Delimiter: Char; DeleteBeforeLoad, WithRownTitle: Boolean);
var
  L1, L2: TStringList;
  i: Integer;
  Auto: Boolean;
  Marker: TMarker;
  Stop: Boolean;
begin
  if not FileExists(FileName) then
    raise Exception.Create(GetTranslateText('El fichero no existe', Language));

  Auto := AutoUpdate;
  AutoUpdate := False;

  if DeleteBeforeLoad then Clear;

  L1 := TStringList.Create;
  L2 := TStringList.Create;
  try
    L1.LoadFromFile(FileName);
    if WithRownTitle then L1.Delete(0);

    L2.Delimiter := Delimiter;
    {$IF CompilerVersion > 16}  
    L2.StrictDelimiter := True;
    {$IFEND}

    for i := 0 to L1.Count - 1 do
    begin
      L2.DelimitedText := L1[i];

      if (LatColumn > L2.Count) or (LngColumn > L2.Count) or
         ((TitleColumn <>1) and (TitleColumn > L2.Count)) then
        raise Exception.Create(GetTranslateText('El nmero de columna es incorrecto', Language));

      if TitleColumn = -1 then Marker := Add
      else Marker := Add(0, 0, L2[TitleColumn]);
      Marker.Position.Lat := Marker.Position.StringToReal(L2[LatColumn]);
      Marker.Position.Lng := Marker.Position.StringToReal(L2[LngColumn]);

      Stop := False;
      if Assigned(FOnLoadFile) then FOnLoadFile(Self, Marker, i, L1.Count, Stop);
      if Stop then Break;
    end;

    if Auto then AutoUpdate := True;
    ShowElements;
    if Assigned(FAfterLoadFile) then FAfterLoadFile(Self, i, L1.Count);
  finally
    AutoUpdate := Auto;
    if Assigned(L1) then FreeAndNil(L1);
    if Assigned(L2) then FreeAndNil(L2);
  end;
end;

procedure TGMMarker.ShowElements;
var
  i: Integer;
begin
  if not ExecuteScript('DeleteObjects', IntToStr(IdxList)) then Exit;

  for i:= 0 to VisualObjects.Count - 1 do
    Items[i].ChangeProperties;
end;

procedure TGMMarker.EventFired(EventType: TEventType; Params: array of const);
var
  LL: TLatLng;
begin
  inherited;

  if EventType = etInfoWinCloseClick then Exit;

  if High(Params) <> 2 then
    raise Exception.Create(GetTranslateText('Nmero de parmetros incorrecto', Map.Language));

  if (Params[0].VType <> vtExtended) or (Params[1].VType <> vtExtended) then
    raise Exception.Create(GetTranslateText('Tipo de parmetro incorrecto', Map.Language));

  if Params[2].VType <> vtInteger then
    raise Exception.Create(GetTranslateText('Tipo de parmetro incorrecto', Map.Language));

  if Params[2].VInteger > VisualObjects.Count - 1 then
    raise Exception.Create(GetTranslateText('Valor de parmetro incorrecto', Map.Language));

  LL := TLatLng.Create(ControlPrecision(Params[0].VExtended^, Map.Precision),
                       ControlPrecision(Params[1].VExtended^, Map.Precision));
  try
    case EventType of
      etMarkerClick: if Assigned(FOnClick) then FOnClick(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
      etMarkerDblClick: if Assigned(FOnDblClick) then FOnDblClick(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
      etMarkerDrag: if Assigned(FOnDrag) then FOnDrag(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
      etMarkerDragEnd:
      begin
        Items[Params[2].VInteger].FIsUpdating := True;
        Items[Params[2].VInteger].Position.Assign(LL);
        if Assigned(FOnDragEnd) then FOnDragEnd(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
        Items[Params[2].VInteger].FIsUpdating := False;
      end;
      etMarkerDragStart: if Assigned(FOnDragStart) then FOnDragStart(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
      etMarkerMouseDown: if Assigned(FOnMouseDown) then FOnMouseDown(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
      etMarkerMouseOut: if Assigned(FOnMouseOut) then FOnMouseOut(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
      etMarkerMouseOver: if Assigned(FOnMouseOver) then FOnMouseOver(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
      etMarkerMouseUp: if Assigned(FOnMouseUp) then FOnMouseUp(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
      etMarkerRightClick: if Assigned(FOnRightClick) then FOnRightClick(Self, LL, Params[2].VInteger, Items[Params[2].VInteger]);
    end;
  finally
    FreeAndNil(LL);
  end;
end;

{ TMarkers }

function TMarkers.Add: TMarker;
begin
  Result := TMarker(inherited Add);
end;

function TMarkers.GetItems(I: Integer): TMarker;
begin
  Result := TMarker(inherited Items[I]);
end;

function TMarkers.GetOwner: TPersistent;
begin
  Result := TGMMarker(inherited GetOwner);
end;

function TMarkers.Insert(Index: Integer): TMarker;
begin
  Result := TMarker(inherited Insert(Index));
end;

procedure TMarkers.SetItems(I: Integer; const Value: TMarker);
begin
  inherited SetItem(I, Value);
end;

{ TAnimation }

procedure TAnimation.Assign(Source: TPersistent);
begin
  if Source is TAnimation then
  begin
    OnDrop := TAnimation(Source).OnDrop;
    Bounce := TAnimation(Source).Bounce;
  end
  else
    inherited Assign(Source);
end;

constructor TAnimation.Create(aOwner: TMarker);
begin
  FMarker := aOwner;
  FOnDrop := False;
  FBounce := False;

  if Assigned(FMarker) then FIdxList := FMarker.IdxList;
end;

procedure TAnimation.SetBounce(const Value: Boolean);
const
  StrParams = '%s,%s,%s';
var
  Params: String;
begin
  if FBounce = Value then Exit;

  FBounce := Value;

  if Assigned(FMarker) and (FMarker is TMarker) and
     Assigned(FMarker.Collection) and (FMarker.Collection is TMarkers) and
     Assigned(TMarkers(FMarker.Collection).FGMLinkedComponent) and
     TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).AutoUpdate then
  begin
    Params := Format(StrParams, [
                      IntToStr(FIdxList),
                      LowerCase(BoolToStr(FBounce, True)),
                      IntToStr(FMarker.Index)]);
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).ExecuteScript('MarkerSetBounce', Params);

    // control de errores // error control
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).ErrorControl;
  end;
end;

procedure TAnimation.SetOnDrop(const Value: Boolean);
begin
  if FOnDrop = Value then Exit;

  FOnDrop := Value;

  if Assigned(FMarker) and (FMarker is TMarker) and
     Assigned(FMarker.Collection) and (FMarker.Collection is TMarkers) and
     Assigned(TMarkers(FMarker.Collection).FGMLinkedComponent) and
     TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).AutoUpdate then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).ShowElements;
end;

{ TMarker }

procedure TMarker.Assign(Source: TPersistent);
begin
  inherited;

  if Source is TMarker then
  begin
    Animation.Assign(TMarker(Source).Animation);
    Clickable := TMarker(Source).Clickable;
    Draggable := TMarker(Source).Draggable;
    Flat := TMarker(Source).Flat;
    Optimized := TMarker(Source).Optimized;
    Position.Assign(TMarker(Source).Position);
    RaiseOnDrag := TMarker(Source).RaiseOnDrag;
    Title := TMarker(Source).Title;
    Visible := TMarker(Source).Visible;
    MarkerType := TMarker(Source).MarkerType;
    ColoredMarker.Assign(TMarker(Source).ColoredMarker);
    StyledMarker.Assign(TMarker(Source).StyledMarker);
  end;
end;

procedure TMarker.CenterMapToMarker;
begin
  if Assigned(Collection) and (Collection is TMarkers) and
     Assigned(TMarkers(Collection).FGMLinkedComponent) and
     Assigned(TMarkers(Collection).FGMLinkedComponent.Map) then
    TMarkers(Collection).FGMLinkedComponent.Map.SetCenter(Position.Lat, Position.Lng);
end;

function TMarker.ChangeProperties: Boolean;
const
  StrParams = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s';
var
  Params: string;
  Icon: string;
begin
  inherited;

  Result := False;

  if not Assigned(Collection) or not(Collection is TMarkers) or
     not Assigned(TMarkers(Collection).FGMLinkedComponent) or
     not TGMMarker(TMarkers(Collection).FGMLinkedComponent).AutoUpdate or
     not Assigned(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map) or
     (csDesigning in TGMMarker(TMarkers(Collection).FGMLinkedComponent).ComponentState) then
    Exit;

  if MarkerType = mtColored then
    Icon := Format(StrColored, [ColoredMarker.Width,
                                ColoredMarker.Height,
                                StringReplace(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map.TColorToStr(ColoredMarker.CornerColor), '#', '', [rfReplaceAll]),
                                StringReplace(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map.TColorToStr(ColoredMarker.PrimaryColor), '#', '', [rfReplaceAll]),
                                StringReplace(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map.TColorToStr(ColoredMarker.StrokeColor), '#', '', [rfReplaceAll])
                                ])
  else
  begin
    Icon := FIcon;
    if FileExists(Icon) then
    begin
      Icon := 'file:///' + Icon;
      Icon := StringReplace(Icon, '\', '/', [rfReplaceAll]);
    end;
  end;

  Params := Format(StrParams, [
                  IntToStr(IdxList),
                  LowerCase(BoolToStr(Animation.OnDrop, True)),
                  LowerCase(BoolToStr(Animation.Bounce, True)),
                  LowerCase(BoolToStr(Clickable, True)),
                  LowerCase(BoolToStr(Draggable, True)),
                  LowerCase(BoolToStr(Flat, True)),
                  LowerCase(BoolToStr(Optimized, True)),
                  Position.LatToStr(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map.Precision),
                  Position.LngToStr(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map.Precision),
                  LowerCase(BoolToStr(RaiseOnDrag, True)),
                  QuotedStr(TGMMarker(TMarkers(Collection).FGMLinkedComponent).GetConvertedString(Title)),
                  LowerCase(BoolToStr(Visible, True)),
                  QuotedStr(Icon),
                  IntToStr(Index),
                  QuotedStr(InfoWindow.GetConvertedString),
                  LowerCase(BoolToStr(InfoWindow.DisableAutoPan, True)),
                  IntToStr(InfoWindow.MaxWidth),
                  IntToStr(InfoWindow.PixelOffset.Height),
                  IntToStr(InfoWindow.PixelOffset.Width),
                  LowerCase(BoolToStr(InfoWindow.CloseOtherBeforeOpen, True)),
                  QuotedStr(TTransform.MarkerTypeToStr(MarkerType)),
                  QuotedStr(TTransform.StyledIconToStr(StyledMarker.StyledIcon)),
                  QuotedStr(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map.TColorToStr(StyledMarker.BackgroundColor)),
                  QuotedStr(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map.TColorToStr(StyledMarker.TextColor)),
                  QuotedStr(TGMMarker(TMarkers(Collection).FGMLinkedComponent).Map.TColorToStr(StyledMarker.StarColor)),
                  LowerCase(BoolToStr(StyledMarker.ShowStar, True)),
                  LowerCase(BoolToStr(ShowInfoWinMouseOver, True))
                  ]);

  Result := TGMMarker(TMarkers(Collection).FGMLinkedComponent).ExecuteScript('MakeMarker', Params);
  TGMMarker(TMarkers(Collection).FGMLinkedComponent).ErrorControl;
end;

constructor TMarker.Create(Collection: TCollection);
begin
  inherited;

  FAnimation := TAnimation.Create(Self);
  FClickable := True;
  FDraggable := False;
  FFlat := False;
  FOptimized := True;
  FPosition := TLatLng.Create;
  FPosition.OnChange := OnLatLngChange;
  FRaiseOnDrag := True;
  FTitle := '';
  FVisible := True;
  FIsUpdating := False;
  FMarkerType := mtStandard;
  FColoredMarker := TColoredMarker.Create(Self);
  FStyledMarker := TStyledMarker.Create(Self);
end;

destructor TMarker.Destroy;
begin
  if Assigned(FPosition) then FreeAndNil(FPosition);
  if Assigned(FAnimation) then FreeAndNil(FAnimation);
  if Assigned(FColoredMarker) then FreeAndNil(FColoredMarker);
  if Assigned(FStyledMarker) then FreeAndNil(FStyledMarker);

  inherited;
end;

function TMarker.GetDisplayName: string;
begin
  if Length(Title) > 0 then
  begin
    if Length(Title) > 15 then
      Result := Copy(Title, 0, 12) + '...'
    else
      Result := Title;
  end
  else
    Result := inherited GetDisplayName;
end;

function TMarker.GetZIndex: Integer;
begin
  Result := Index;
end;

procedure TMarker.OnLatLngChange(Sender: TObject);
begin
  if FIsUpdating then Exit;

  ChangeProperties;
  if Assigned(TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnPositionChange) then
    TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnPositionChange(
                  TGMMarker(TMarkers(Collection).FGMLinkedComponent),
                  Position,
                  Index,
                  Self);
end;

procedure TMarker.SetClickable(const Value: Boolean);
begin
  if FClickable = Value then Exit;

  FClickable := Value;

  ChangeProperties;
  if Assigned(TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnClickableChange) then
    TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnClickableChange(
                  TGMMarker(TMarkers(Collection).FGMLinkedComponent),
                  Index,
                  Self);
end;

procedure TMarker.SetDraggable(const Value: Boolean);
begin
  if FDraggable = Value then Exit;

  FDraggable := Value;

  ChangeProperties;
  if Assigned(TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnDragableChange) then
    TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnDragableChange(
                  TGMMarker(TMarkers(Collection).FGMLinkedComponent),
                  Index,
                  Self);
end;

procedure TMarker.SetFlat(const Value: Boolean);
begin
  if FFlat = Value then Exit;

  FFlat := Value;

  ChangeProperties;
  if Assigned(TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnFlatChange) then
    TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnFlatChange(
                  TGMMarker(TMarkers(Collection).FGMLinkedComponent),
                  Index,
                  Self);
end;

procedure TMarker.SetIcon(const Value: string);
begin
  if FIcon = Value then Exit;

  FIcon := Value;

  ChangeProperties;
  if Assigned(TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnIconChange) then
    TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnIconChange(
                  TGMMarker(TMarkers(Collection).FGMLinkedComponent),
                  Index,
                  Self);
end;

procedure TMarker.SetIdxList(const Value: Cardinal);
begin
  inherited;

  FAnimation.IdxList := Value;
end;

procedure TMarker.SetMarkerType(const Value: TMarkerType);
begin
  if FMarkerType = Value then Exit;

  FMarkerType := Value;
  ChangeProperties;
end;

procedure TMarker.SetOptimized(const Value: Boolean);
begin
  if FOptimized = Value then Exit;

  FOptimized := Value;

  ChangeProperties;
end;

procedure TMarker.SetRaiseOnDrag(const Value: Boolean);
begin
  if FRaiseOnDrag = Value then Exit;

  FRaiseOnDrag := Value;

  ChangeProperties;
end;

procedure TMarker.SetTitle(const Value: string);
begin
  if FTitle = Value then Exit;

  FTitle := Value;

  ChangeProperties;
  if Assigned(TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnTitleChange) then
    TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnTitleChange(
                  TGMMarker(TMarkers(Collection).FGMLinkedComponent),
                  Index,
                  Self);
end;

procedure TMarker.SetVisible(const Value: Boolean);
begin
  if FVisible = Value then Exit;

  FVisible := Value;

  ChangeProperties;
  if Assigned(TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnVisibleChange) then
    TGMMarker(TMarkers(Collection).FGMLinkedComponent).FOnVisibleChange(
                  TGMMarker(TMarkers(Collection).FGMLinkedComponent),
                  Index,
                  Self);
end;

{ TColoredMarker }

procedure TColoredMarker.Assign(Source: TPersistent);
begin
  if Source is TColoredMarker then
  begin
    FMarker := TColoredMarker(Source).FMarker;
    Height := TColoredMarker(Source).Height;
    Width := TColoredMarker(Source).Width;
    CornerColor := TColoredMarker(Source).CornerColor;
    StrokeColor := TColoredMarker(Source).StrokeColor;
    PrimaryColor := TColoredMarker(Source).PrimaryColor;
  end
  else inherited;
end;

constructor TColoredMarker.Create(aOwner: TMarker);
begin
  FMarker := aOwner;
  FHeight := 32;
  FWidth := 32;
  FStrokeColor := clBlack;
  FCornerColor := clWhite;
  FPrimaryColor := clRed;
end;

procedure TColoredMarker.SetCornerColor(const Value: TColor);
begin
  if FCornerColor = Value then Exit;

  FCornerColor := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

procedure TColoredMarker.SetHeight(const Value: Integer);
begin
  if FHeight = Value then Exit;

  FHeight := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

procedure TColoredMarker.SetPrimaryColor(const Value: TColor);
begin
  if FPrimaryColor = Value then Exit;

  FPrimaryColor := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

procedure TColoredMarker.SetStrokeColor(const Value: TColor);
begin
  if FStrokeColor = Value then Exit;

  FStrokeColor := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

procedure TColoredMarker.SetWidth(const Value: Integer);
begin
  if FWidth = Value then Exit;

  FWidth := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnColoredMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

{ TStyledMarker }

procedure TStyledMarker.Assign(Source: TPersistent);
begin
  if Source is TStyledMarker then
  begin
    FMarker := TStyledMarker(Source).FMarker;
    StyledIcon := TStyledMarker(Source).StyledIcon;
    BackgroundColor := TStyledMarker(Source).BackgroundColor;
    TextColor := TStyledMarker(Source).TextColor;
    StarColor := TStyledMarker(Source).StarColor;
    ShowStar := TStyledMarker(Source).ShowStar;
  end
  else inherited;
end;

constructor TStyledMarker.Create(aOwner: TMarker);
begin
  FMarker := aOwner;
  FStyledIcon := siMarker;
  FBackgroundColor := clRed;
  FTextColor := clBlack;
  FStarColor := clLime;
  FShowStar := False;
end;

procedure TStyledMarker.SetBackgroundColor(const Value: TColor);
begin
  if FBackgroundColor = Value then Exit;

  FBackgroundColor := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

procedure TStyledMarker.SetShowStar(const Value: Boolean);
begin
  if FShowStar = Value then Exit;

  FShowStar := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

procedure TStyledMarker.SetStarColor(const Value: TColor);
begin
  if FStarColor = Value then Exit;

  FStarColor := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

procedure TStyledMarker.SetStyledIcon(const Value: TStyledIcon);
begin
  if FStyledIcon = Value then Exit;

  FStyledIcon := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

procedure TStyledMarker.SetTextColor(const Value: TColor);
begin
  if FTextColor = Value then Exit;

  FTextColor := Value;

  if not Assigned(FMarker) then Exit;

  FMarker.ChangeProperties;
  if Assigned(TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange) then
    TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent).FOnStyledMarkerChange(
                  TGMMarker(TMarkers(FMarker.Collection).FGMLinkedComponent),
                  FMarker.Index,
                  FMarker);
end;

{ TStyleLabel }

procedure TStyleLabel.Assign(Source: TPersistent);
begin
  if Source is TStyleLabel then
  begin
    FMarker := TStyleLabel(Source).FMarker;
    Font.Assign(TStyleLabel(Source).Font);
    Border.Assign(TStyleLabel(Source).Border);
    Opacity := TStyleLabel(Source).Opacity;
    Background := TStyleLabel(Source).Background;
    NoBackground := TStyleLabel(Source).NoBackground;
  end
  else inherited;
end;

constructor TStyleLabel.Create(aOwner: TMarker);
begin
  FMarker := aOwner;
  FFont := TGMFont.Create(FMarker);
  FBorder := TGMBorder.Create(FMarker);
  FOpacity := 1;
  FBackground := clWhite;
  FNoBackground := False;
end;

destructor TStyleLabel.Destroy;
begin
  if Assigned(FFont) then FreeAndNil(FFont);
  if Assigned(FBorder) then FreeAndNil(FBorder);

  inherited;
end;

procedure TStyleLabel.SetBackground(const Value: TColor);
begin
  FBackground := Value;
end;

procedure TStyleLabel.SetNoBackground(const Value: Boolean);
begin
  FNoBackground := Value;
end;

procedure TStyleLabel.SetOpacity(const Value: Real);
begin
  FOpacity := Value;
  if FOpacity < 0 then FOpacity := 0;
  if FOpacity > 1 then FOpacity := 1;
  FOpacity := Trunc(FOpacity * 100) / 100;
end;

{ TGMFont }

procedure TGMFont.Assign(Source: TPersistent);
begin
  if Source is TGMFont then
  begin
    FMarker := TGMFont(Source).FMarker;
    Style := TGMFont(Source).Style;
    Size := TGMFont(Source).Size;
    Color := TGMFont(Source).Color;
  end
  else inherited;
end;

constructor TGMFont.Create(aOwner: TMarker);
begin
  FMarker := aOwner;
  FStyle := [fsBold];
  FSize := 20;
  FColor := clBlack;
end;

procedure TGMFont.SetColor(const Value: TColor);
begin
  FColor := Value;
end;

procedure TGMFont.SetSize(const Value: Integer);
begin
  FSize := Value;
end;

procedure TGMFont.SetStyle(const Value: TGMFontStyles);
begin
  FStyle := Value;
end;

{ TGMBorder }

procedure TGMBorder.Assign(Source: TPersistent);
begin
  if Source is TGMBorder then
  begin
    FMarker := TGMBorder(Source).FMarker;
    Size := TGMBorder(Source).Size;
    Color := TGMBorder(Source).Color;
    Style := TGMBorder(Source).Style;
  end
  else inherited;
end;

constructor TGMBorder.Create(aOwner: TMarker);
begin
  FMarker := aOwner;
  FSize := 2;
  FColor := clBlack;
  FStyle := bsSolid;
end;

procedure TGMBorder.SetColor(const Value: TColor);
begin
  FColor := Value;
end;

procedure TGMBorder.SetSize(const Value: Integer);
begin
  FSize := Value;
end;

procedure TGMBorder.SetStyle(const Value: TBorderStyle);
begin
  FStyle := Value;
end;

end.
